#!/usr/bin/python
#
# Copyright 2009, Douglas Greiman
#
# Biomedin 217, Winter 2008-2009, Final Project
# Student name: Douglas Greiman
# SUNet id: duggelz
# Student id: 005512989
#
# This program queries the Rat Genome Database (RGD) for all the genes
# and Gene Ontology annotations present in a particular range of
# locations in the rat genome.

# Module imports
import csv
import pprint
import time
import urllib
import urllib2


# URL to RGD Biomart REST interface
RGD_BIOMART_URL = 'http://www.biomart.org/biomart/martservice'

# Template used to construct request to RGB BioMart
XML_QUERY_DOC_TEMPLATE="""\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query  virtualSchemaName = "default" header = "0" uniqueRows = "0"
        count = "" datasetConfigVersion = "0.4" formatter = "CSV" >
	<Dataset name = "rgd_genes" interface = "default" >
%(attribute_tags)s
	     	<ValueFilter name = "end_all" value = "%(end_all)s"/>
	     	<ValueFilter name = "chromosome_34_list_filter" value = "%(chromosome)s"/>
	     	<ValueFilter name = "start_all" value = "%(start_all)s"/>
	  </Dataset>
</Query>
"""

# For debugging
SAMPLE_XML_RESPONSE = """Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0005509,calcium ion binding,1600115,IEA,InterPro:IPR002126,F,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0007156,homophilic cell adhesion,1600115,IEA,InterPro:IPR002126,P,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0016020,membrane,1600115,IEA,InterPro:IPR002126,C,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0005515,protein binding,1624291,ISO,RGD:1557959,F,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0043231,intracellular membrane-bounded organelle,1624291,ISO,RGD:1353305,C,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0005730,nucleolus,1624291,ISO,RGD:1353305,C,,,,,,,,,,,,,,,,,,,,,,,
Pcdh11x,protocadherin 11 X-linked ,1562864,317204,,,X,+,109834025,110557620,protein-coding,v3.4,,GO:0005737,cytoplasm,1624291,ISO,RGD:1353305,C,,,,,,,,,,,,,,,,,,,,,,,""".splitlines()

# List of all attributes recognized by RGD Biomart, for reference only
ALL_ATTRIBUTE_NAMES = [
"symbol",
"name",
"rgd_id",
"entrez_gene_id",
"description",
"map34_mapped_repseq",
"map34_chromosome",
"map34_strand",
"map34_start",
"map34_stop",
"map34_score",
"map34_version",
"qualifier",
"go_id",
"go_term",
"db_reference",
"evidence",
"with_from",
"aspect",
"slim_term",
"slim_id",
"do_db_reference",
"do_id",
"do_term",
"do_aspect",
"do_evidence",
"do_qualifier",
"do_with_from",
"mp_evidence",
"mp_qualifier",
"mp_with_from",
"pw_evidence",
"pw_qualifier",
"pw_with_from",
"pw_aspect",
"pw_id",
"pw_term",
"pw_db_reference",
"mp_aspect",
"mp_term",
"mp_id",
"mp_db_reference",
]

# List of attributes we actually query
ATTRIBUTE_NAMES = [
    "symbol",
    "name",
    "rgd_id",
    "entrez_gene_id",
    "description",
    "go_id",
    "go_term",
    ]


def query(chromosome, location, span=10000):
    """Query the RGD for information on a particular nucleotide range.

    chromosome: 1 to 20, X, Y
    location: 0 to length of chromosome, integer
    span: Look up to this many base pairs before and after 'location'
    
    """

    # Construct XML request
    attribute_names = ATTRIBUTE_NAMES
    attribute_tags = ['<Attribute name = "%s" />' % a for a in attribute_names]
    xml_vars = {
        'end_all': location + span,
        'start_all': max(0, location - span),
        'chromosome': chromosome,
        'attribute_tags': '\n'.join(attribute_tags),
        }
    xml_query_doc = XML_QUERY_DOC_TEMPLATE % xml_vars

    # Encode the document contents as the 'query' parameter and send
    # it as part of a POST request to RGD
    post_data = urllib.urlencode([("query", xml_query_doc)])
    start = time.time()
    response = urllib2.urlopen(RGD_BIOMART_URL, post_data)
    end = time.time()
    print "Took %.3f seconds" % (end-start)

    # Parse the response as a CSV document
    response_reader = csv.DictReader(response, fieldnames=attribute_names)
    response_rows = list(response_reader)
    return response_rows
# enddef


if __name__ == '__main__':
    # Run sample query for testing
    print query('X', 110234510)
